@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: grdtrend.c 17449 2017-01-16 21:27:04Z pwessel $
  *
- *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: Reads a grid file and fits a trend surface.  Trend surface
@@ -27,7 +26,7 @@
  *
  * Author:		W. H. F. Smith
  * Date:		1 JAN, 2010
- * Version:	5 API
+ * Version:	6 API
  *
  * Explanations:
  *
@@ -70,7 +69,7 @@
  * the sums which are the elements of the normal equations;
  * while they analytically cancel to zero, the addition errors
  * would likely prevent this.  Therefore we have written a
- * routine, grd_trivial_model(), to handle this case.
+ * routine, grdtrend_grd_trivial_model(), to handle this case.
  *
  * If the problem is more complex than the above trivial case,
  * (missing values, weighted problem, or n_model > 4), then
@@ -81,40 +80,46 @@
  * are conveniently sized near 0 or 1.
  */
 
-#define THIS_MODULE_NAME	"grdtrend"
-#define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Fit trend surface to grids and compute residuals"
-#define THIS_MODULE_KEYS	"<G{,DG),TG),WG)"
-
 #include "gmt_dev.h"
 
-#define GMT_PROG_OPTIONS "-RV"
+#define THIS_MODULE_CLASSIC_NAME	"grdtrend"
+#define THIS_MODULE_MODERN_NAME	"grdtrend"
+#define THIS_MODULE_LIB		"core"
+#define THIS_MODULE_PURPOSE	"Fit trend surface to grids and compute residuals"
+#define THIS_MODULE_KEYS	"<G{,DG),TG),WG(,WG)"
+#define THIS_MODULE_NEEDS	""
+#define THIS_MODULE_OPTIONS "-RV"
+
+#ifdef DOUBLE_PRECISION_GRID
+#define fabsf(x) fabs(x)
+#endif
 
 struct GRDTREND_CTRL {	/* All control options for this program (except common args) */
-	struct In {
+	struct GRDTREND_In {
 		bool active;
 		char *file;
 	} In;
-	struct D {	/* -D<diffgrid> */
+	struct GRDTREND_D {	/* -D<diffgrid> */
 		bool active;
 		char *file;
 	} D;
-	struct N {	/* -N[r]<n_model> */
+	struct GRDTREND_N {	/* -N[r]<n_model> */
 		bool active;
 		bool robust;
 		unsigned int value;
 	} N;
-	struct T {	/* -T<trend.grd> */
+	struct GRDTREND_T {	/* -T<trend.grd> */
 		bool active;
 		char *file;
 	} T;
-	struct W {	/* -W<weight.grd> */
+	struct GRDTREND_W {	/* -W<weight.grd>[+s] */
 		bool active;
+		unsigned int mode;
 		char *file;
 	} W;
 };
 
-GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+static void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct GRDTREND_CTRL *C;
 
 	C = gmt_M_memory (GMT, NULL, 1, struct GRDTREND_CTRL);
@@ -124,7 +129,7 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	return (C);
 }
 
-GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *C) {	/* Deallocate control structure */
+static void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
 	gmt_M_str_free (C->In.file);
 	gmt_M_str_free (C->D.file);
@@ -133,17 +138,17 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *C) {	/* De
 	gmt_M_free (GMT, C);
 }
 
-GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+static int usage (struct GMTAPI_CTRL *API, int level) {
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdtrend <ingrid> -N<n_model>[r] [-D<diffgrid>] [%s]\n", GMT_Rgeo_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-T<trendgrid>] [%s] [-W<weightgrid>]\n\n", GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <ingrid> -N<n_model>[+r] [-D<diffgrid>] [%s]\n", name, GMT_Rgeo_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-T<trendgrid>] [%s] [-W<weightgrid>[+s]] [%s]\n\n", GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Message (API, GMT_TIME_NONE, "\t<ingrid> is name of grid file to fit trend to.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Fit a [robust] model with <n_model> terms.  <n_model> in [1,10].  E.g., robust planar = -N3r.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Model parameters order is given as follows:\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-N Fit a [robust] model with <n_model> terms.  <n_model> in [1,10].  E.g., robust planar = -N3+r.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Model parameters order is given as follows; append +r for robust solution:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   z = m1 + m2*x + m3*y + m4*x*y + m5*x^2 + m6*y^2 + m7*x^3 + m8*x^2*y + m9*x*y^2 + m10*y^3.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Supply filename to write grid file of differences (input - trend).\n");
@@ -153,12 +158,13 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Supply filename if you want to [read and] write grid file of weights.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If <weightgrid> can be read at run, and if robust = false, weighted problem will be solved.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If robust = true, weights used for robust fit will be written to <weightgrid>.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +s to read standard deviations from file and compute weights as 1/s^2.\n");
 	GMT_Option (API, ".");
 
 	return (GMT_MODULE_USAGE);
 }
 
-GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *Ctrl, struct GMT_OPTION *options) {
+static int parse (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *Ctrl, struct GMT_OPTION *options) {
 
 	/* This parses the options provided to grdtrend and sets parameters in Ctrl.
 	 * Note Ctrl has already been initialized and non-zero default values set.
@@ -167,6 +173,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *Ctrl, struct GM
 	 * returned when registering these sources/destinations with the API.
 	 */
 
+	char *c = NULL;
 	unsigned int n_errors = 0, n_files = 0, j;
 	struct GMT_OPTION *opt = NULL;
 
@@ -175,41 +182,47 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *Ctrl, struct GM
 			/* Common parameters */
 
 			case '<':	/* Input file (only one is accepted) */
-				if (n_files++ > 0) break;
-				if ((Ctrl->In.active = gmt_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_GRID)) != 0)
-					Ctrl->In.file = strdup (opt->arg);
-				else
-					n_errors++;
+				if (n_files++ > 0) {n_errors++; continue; }
+				Ctrl->In.active = true;
+				if (opt->arg[0]) Ctrl->In.file = strdup (opt->arg);
+				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->In.file))) n_errors++;
 				break;
 
 			/* Processes program-specific parameters */
 
 			case 'D':
-				if ((Ctrl->D.active = gmt_check_filearg (GMT, 'D', opt->arg, GMT_OUT, GMT_IS_GRID)) != 0)
-					Ctrl->D.file = strdup (opt->arg);
-				else
-					n_errors++;
+				Ctrl->D.active = true;
+				if (opt->arg[0]) Ctrl->D.file = strdup (opt->arg);
+				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->D.file))) n_errors++;
 				break;
 			case 'N':
 				/* Must check for both -N[r]<n_model> and -N<n_model>[r] due to confusion */
 				Ctrl->N.active = true;
 				if (strchr (opt->arg, 'r')) Ctrl->N.robust = true;
-				j = (opt->arg[0] == 'r') ? 1 : 0;
+				j = (opt->arg[0] == 'r') ? 1 : 0;	/* GMT4 backwardness */
 				if (opt->arg[j]) Ctrl->N.value = atoi(&opt->arg[j]);
 				break;
 			case 'T':
-				if ((Ctrl->T.active = gmt_check_filearg (GMT, 'T', opt->arg, GMT_OUT, GMT_IS_GRID)) != 0)
-					Ctrl->T.file = strdup (opt->arg);
-				else
-					n_errors++;
+				Ctrl->T.active = true;
+				if (opt->arg[0]) Ctrl->T.file = strdup (opt->arg);
+				if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->T.file))) n_errors++;
 				break;
 			case 'W':
 				Ctrl->W.active = true;
-				/* OK if this file doesn't exist; we always write to that file on output */
-				if (gmt_check_filearg (GMT, 'W', opt->arg, GMT_IN, GMT_IS_GRID) || gmt_check_filearg (GMT, 'W', opt->arg, GMT_OUT, GMT_IS_GRID))
-					Ctrl->W.file = strdup (opt->arg);
+				if (opt->arg[0] != '+' && (c = strstr (opt->arg, "+s"))) {	/* Gave a file arg plus a +s modifier */
+					Ctrl->W.mode = 2;
+					c[0] = '\0';	/* Chop off modifier */
+				}
 				else
-					n_errors++;
+					Ctrl->W.mode = 1;
+				/* OK if this file doesn't exist; we always write to that file on output */
+				Ctrl->W.file = strdup (opt->arg);
+				if (!gmt_access (GMT, Ctrl->W.file, R_OK)) {	/* FOund the file */
+					if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_IN, GMT_FILE_REMOTE, &(Ctrl->W.file))) n_errors++;
+				}
+				else {
+					if (GMT_Get_FilePath (GMT->parent, GMT_IS_GRID, GMT_OUT, GMT_FILE_LOCAL, &(Ctrl->W.file))) n_errors++;
+				}
 				break;
 
 			default:	/* Report bad options */
@@ -218,14 +231,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *Ctrl, struct GM
 		}
 	}
 
-	n_errors += gmt_M_check_condition (GMT, n_files != 1, "Syntax error: Must specify an input grid file\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->N.value <= 0 || Ctrl->N.value > 10, "Syntax error -N option: Specify 1-10 model parameters\n");
+	n_errors += gmt_M_check_condition (GMT, n_files != 1, "Must specify an input grid file\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->N.value <= 0 || Ctrl->N.value > 10, "Option -N: Specify 1-10 model parameters\n");
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
 #if 0
-GMT_LOCAL void set_up_vals (double *val, unsigned int nval, double vmin, double vmax, double dv, unsigned int pixel_reg) {
+GMT_LOCAL void grdtrend_set_up_vals (double *val, unsigned int nval, double vmin, double vmax, double dv, unsigned int pixel_reg) {
 	/* Store x[i], y[j] once for all to save time  */
 	unsigned int i;
 	double v, middle, drange, true_min, true_max;
@@ -246,9 +259,10 @@ GMT_LOCAL void set_up_vals (double *val, unsigned int nval, double vmin, double
 }
 #endif
 
-GMT_LOCAL void load_pstuff (double *pstuff, unsigned int n_model, double x, double y, unsigned int newx, unsigned int newy) {
+GMT_LOCAL void grdtrend_load_pstuff (double *pstuff, unsigned int n_model, double x, double y, unsigned int newx, unsigned int newy) {
 	/* Compute Legendre polynomials of x[i],y[j] as needed  */
-	/* If either x or y has changed, compute new Legendre polynomials as needed  */
+	/* If either x or y has changed, compute new Legendre polynomials as needed.
+	 * Remember: pstuff[0] == 1 throughout.  */
 
 	if (newx) {
 		if (n_model >= 2) pstuff[1] = x;
@@ -269,20 +283,20 @@ GMT_LOCAL void load_pstuff (double *pstuff, unsigned int n_model, double x, doub
 	return;
 }
 
-GMT_LOCAL void compute_trend (struct GMT_CTRL *GMT, struct GMT_GRID *T, double *xval, double *yval, double *gtd, unsigned int n_model, double *pstuff) {
+GMT_LOCAL void grdtrend_compute_trend (struct GMT_CTRL *GMT, struct GMT_GRID *T, double *xval, double *yval, double *gtd, unsigned int n_model, double *pstuff) {
 	/* Find trend from a model  */
 	unsigned int row, col, k;
 	uint64_t ij;
 	gmt_M_unused(GMT);
 
 	gmt_M_grd_loop (GMT, T, row, col, ij) {
-		load_pstuff (pstuff, n_model, xval[col], yval[row], 1, (!(col)));
+		grdtrend_load_pstuff (pstuff, n_model, xval[col], yval[row], 1, (!(col)));
 		T->data[ij] = 0.0f;
-		for (k = 0; k < n_model; k++) T->data[ij] += (float)(pstuff[k]*gtd[k]);
+		for (k = 0; k < n_model; k++) T->data[ij] += (gmt_grdfloat)(pstuff[k]*gtd[k]);
 	}
 }
 
-GMT_LOCAL void compute_resid (struct GMT_CTRL *GMT, struct GMT_GRID *D, struct GMT_GRID *T, struct GMT_GRID *R) {
+GMT_LOCAL void grdtrend_compute_resid (struct GMT_CTRL *GMT, struct GMT_GRID *D, struct GMT_GRID *T, struct GMT_GRID *R) {
 	/* Find residuals from a trend  */
 	unsigned int row, col;
 	uint64_t ij;
@@ -291,7 +305,7 @@ GMT_LOCAL void compute_resid (struct GMT_CTRL *GMT, struct GMT_GRID *D, struct G
 	gmt_M_grd_loop (GMT, T, row, col, ij) R->data[ij] = D->data[ij] - T->data[ij];
 }
 
-GMT_LOCAL void grd_trivial_model (struct GMT_CTRL *GMT, struct GMT_GRID *G, double *xval, double *yval, double *gtd, unsigned int n_model) {
+GMT_LOCAL void grdtrend_grd_trivial_model (struct GMT_CTRL *GMT, struct GMT_GRID *G, double *xval, double *yval, double *gtd, unsigned int n_model) {
 	/* Routine to fit up elementary polynomial model of grd data,
 	model = gtd[0] + gtd[1]*x + gtd[2]*y + gtd[3] * x * y,
 	where x,y are normalized to range [-1,1] and there are no
@@ -332,7 +346,7 @@ GMT_LOCAL void grd_trivial_model (struct GMT_CTRL *GMT, struct GMT_GRID *G, doub
 	return;
 }
 
-GMT_LOCAL double compute_chisq (struct GMT_CTRL *GMT, struct GMT_GRID *R, struct GMT_GRID *W, double scale) {
+GMT_LOCAL double grdtrend_compute_chisq (struct GMT_CTRL *GMT, struct GMT_GRID *R, struct GMT_GRID *W, double scale) {
 	/* Find Chi-Squared from weighted residuals  */
 	unsigned int row, col;
 	uint64_t ij;
@@ -350,11 +364,11 @@ GMT_LOCAL double compute_chisq (struct GMT_CTRL *GMT, struct GMT_GRID *R, struct
 	return (chisq);
 }
 
-GMT_LOCAL double compute_robust_weight (struct GMT_CTRL *GMT, struct GMT_GRID *R, struct GMT_GRID *W) {
+GMT_LOCAL double grdtrend_compute_robust_weight (struct GMT_CTRL *GMT, struct GMT_GRID *R, struct GMT_GRID *W) {
 	/* Find weights from residuals  */
 	unsigned int row, col;
 	uint64_t j = 0, j2, ij;
-	float r, mad, scale;
+	gmt_grdfloat r, mad, scale;
 
 	gmt_M_grd_loop (GMT, R, row, col, ij) {
 		if (gmt_M_is_fnan (R->data[ij])) continue;
@@ -372,7 +386,7 @@ GMT_LOCAL double compute_robust_weight (struct GMT_CTRL *GMT, struct GMT_GRID *R
 
 	/* Use weight according to Huber (1981), but squared */
 
-	gmt_M_memset (W->data, W->header->size, float);	/* Wipe W clean */
+	gmt_M_memset (W->data, W->header->size, gmt_grdfloat);	/* Wipe W clean */
 	gmt_M_grd_loop (GMT, R, row, col, ij) {
 		if (gmt_M_is_fnan (R->data[ij])) {
 			W->data[ij] = R->data[ij];
@@ -385,7 +399,7 @@ GMT_LOCAL double compute_robust_weight (struct GMT_CTRL *GMT, struct GMT_GRID *R
 	return (scale);
 }
 
-GMT_LOCAL void write_model_parameters (struct GMT_CTRL *GMT, double *gtd, unsigned int n_model) {
+GMT_LOCAL void grdtrend_write_model_parameters (struct GMT_CTRL *GMT, double *gtd, unsigned int n_model) {
 	/* Do reports if gmtdefs.verbose = NORMAL or above  */
 	unsigned int i;
 	char pbasis[10][16], format[GMT_BUFSIZ];
@@ -402,12 +416,12 @@ GMT_LOCAL void write_model_parameters (struct GMT_CTRL *GMT, double *gtd, unsign
 	sprintf (pbasis[9], "P3(y)");
 
 	sprintf(format, "Coefficient fit to %%s: %s\n", GMT->current.setting.format_float_out);
-	for (i = 0; i < n_model; i++) GMT_Message (GMT->parent, GMT_TIME_NONE, format, pbasis[i], gtd[i]);
+	for (i = 0; i < n_model; i++) GMT_Report (GMT->parent, GMT_MSG_INFORMATION, format, pbasis[i], gtd[i]);
 
 	return;
 }
 
-GMT_LOCAL void load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, unsigned int n_model, struct GMT_GRID *W, bool weighted) {
+GMT_LOCAL void grdtrend_load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, unsigned int n_model, struct GMT_GRID *W, bool weighted) {
 	/* Routine to load the matrix G'G (gtg) and vector G'd (gtd)
 	for the normal equations.  Routine uses indices i,j to refer
 	to the grid file of data, and k,l to refer to the k_row, l_col
@@ -425,23 +439,23 @@ GMT_LOCAL void load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, doubl
 	gmt_M_memset (gtd, n_model, double);
 	gmt_M_memset (gtg, n_model * n_model, double);
 
-	/* Now get going.  Have to load_pstuff separately in i and j,
+	/* Now get going.  Have to grdtrend_load_pstuff separately in i and j,
 	   because it is possible that we skip data when i = 0.
 	   Loop over all data */
 
 	gmt_M_row_loop (GMT, G, row) {
-		load_pstuff (pstuff, n_model, xval[0], yval[row], 0, 1);
+		grdtrend_load_pstuff (pstuff, n_model, xval[0], yval[row], 0, 1);
 		gmt_M_col_loop (GMT, G, row, col, ij) {
 
 			if (gmt_M_is_fnan (G->data[ij]))continue;
 
 			n_used++;
-			load_pstuff (pstuff, n_model, xval[col], yval[row], 1, 0);
+			grdtrend_load_pstuff (pstuff, n_model, xval[col], yval[row], 1, 0);
 
 			if (weighted) {
 				/* Loop over all gtg and gtd elements */
-				gtd[0] += (G->data[ij] * W->data[ij]);
-				gtg[0] += W->data[ij];
+				gtd[0] += (G->data[ij] * W->data[ij]);	/* Implicitly multiply by pstuff[0] which is 1 */
+				gtg[0] += W->data[ij];			/* Implicitly multiply by pstuff[0] which is 1 */
 				for (k = 1; k < n_model; k++) {
 					gtd[k] += (G->data[ij] * W->data[ij] * pstuff[k]);
 					gtg[k] += (W->data[ij] * pstuff[k]);
@@ -450,7 +464,7 @@ GMT_LOCAL void load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, doubl
 			}
 			else {	/* If !weighted  */
 				/* Loop over all gtg and gtd elements */
-				gtd[0] += G->data[ij];
+				gtd[0] += G->data[ij];	/* Implicitly multiply by pstuff[0] which is 1 */
 				for (k = 1; k < n_model; k++) {
 					gtd[k] += (G->data[ij] * pstuff[k]);
 					gtg[k] += pstuff[k];
@@ -475,12 +489,12 @@ GMT_LOCAL void load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, doubl
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-int GMT_grdtrend (void *V_API, int mode, void *args) {
+EXTERN_MSC int GMT_grdtrend (void *V_API, int mode, void *args) {
 	/* High-level function that implements the grdcontour task */
 
-	bool trivial, weighted,iterations, set_ones = true;
+	bool trivial, weighted, set_ones = true;
 	int error = 0;
-	unsigned int row, col;
+	unsigned int row, col, iterations;
 
 	uint64_t ij;
 
@@ -507,26 +521,28 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
-	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the grdtrend main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid\n");
+	GMT_Report (API, GMT_MSG_INFORMATION, "Processing input grid\n");
 	weighted = (Ctrl->N.robust || Ctrl->W.active);
 	trivial = (Ctrl->N.value < 5 && !weighted);
 
 	gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Current -R setting, if any */
-	if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_HEADER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) {
+	if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) {
 		Return (API->error);
 	}
-	if (gmt_M_is_subset (GMT, G->header, wesn)) gmt_M_err_fail (GMT, gmt_adjust_loose_wesn (GMT, wesn, G->header), "");	/* Subset requested; make sure wesn matches header spacing */
-	if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_DATA_ONLY, wesn, Ctrl->In.file, G) == NULL) {	/* Get subset */
+	if (gmt_M_is_subset (GMT, G->header, wesn)) {	/* Subset requested; make sure wesn matches header spacing */
+		if ((error = gmt_M_err_fail (GMT, gmt_adjust_loose_wesn (GMT, wesn, G->header), "")))
+			Return (error);
+	}
+	if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn, Ctrl->In.file, G) == NULL) {	/* Get subset */
 		Return (API->error);
 	}
 
@@ -552,23 +568,31 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 
 	if (weighted) {
 		if (!gmt_access (GMT, Ctrl->W.file, R_OK)) {	/* We have weights on input  */
-			if ((W = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_HEADER_ONLY, NULL, Ctrl->W.file, NULL)) == NULL) {	/* Get header only */
-				gmt_M_free (GMT, gtd);		gmt_M_free (GMT, gtg);
-				gmt_M_free (GMT, xval);		gmt_M_free (GMT, yval);
-				gmt_M_free (GMT, old);		gmt_M_free (GMT, pstuff);
-				Return (API->error);
+			if ((W = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->W.file, NULL)) == NULL) {	/* Get header only */
+				error = API->error;
+				goto END;
 			}
 			if (W->header->n_columns != G->header->n_columns || W->header->n_rows != G->header->n_rows)
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: Input weight file does not match input data file.  Ignoring.\n");
+				GMT_Report (API, GMT_MSG_ERROR, "Input weight file does not match input data file.  Ignoring.\n");
 			else {
-				if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_DATA_ONLY, NULL, Ctrl->W.file, W) == NULL) {	/* Get data */
-					Return (API->error);
+				if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, NULL, Ctrl->W.file, W) == NULL) {	/* Get data */
+					error = API->error;
+					goto END;
+				}
+				gmt_M_str_free (Ctrl->W.file);	/* Prevent that the weights grid is overwritten later down */
+				if (Ctrl->W.mode == 2) {	/* Convert sigmas to weights */
+					gmt_M_grd_loop (GMT, W, row, col, ij) {
+						W->data[ij] = (gmt_grdfloat)(1.0 / (W->data[ij] * W->data[ij]));
+					}
 				}
 				set_ones = false;
 			}
 		}
 		if (set_ones) {
-			if ((W = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, G)) == NULL) Return (API->error);	/* Pointer for grid with unit weights  */
+			if ((W = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, G)) == NULL) { /* Pointer for grid with unit weights  */
+				error = API->error;
+				goto END;
+			}
 			gmt_M_setnval (W->data, W->header->size, 1.0f);
 		}
 	}
@@ -586,47 +610,49 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 	/* Do the problem */
 
 	if (trivial) {
-		grd_trivial_model (GMT, G, xval, yval, gtd, Ctrl->N.value);
-		compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
-		if (Ctrl->D.active) compute_resid (GMT, G, T, R);
+		grdtrend_grd_trivial_model (GMT, G, xval, yval, gtd, Ctrl->N.value);
+		grdtrend_compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
+		if (Ctrl->D.active) grdtrend_compute_resid (GMT, G, T, R);
 	}
 	else {	/* Problem is not trivial  !!  */
 		int ierror;
-		load_gtg_and_gtd (GMT, G, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, W, weighted);
+		grdtrend_load_gtg_and_gtd (GMT, G, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, W, weighted);
 		ierror = gmt_gauss (GMT, gtg, gtd, Ctrl->N.value, Ctrl->N.value, true);
 		if (ierror) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Gauss returns error code %d\n", ierror);
-			return (GMT_RUNTIME_ERROR);
+			GMT_Report (API, GMT_MSG_ERROR, "Gauss returns error code %d\n", ierror);
+			error = GMT_RUNTIME_ERROR;
+			goto END;
 		}
-		compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
-		if (Ctrl->D.active || Ctrl->N.robust) compute_resid (GMT, G, T, R);
+		grdtrend_compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
+		if (Ctrl->D.active || Ctrl->N.robust) grdtrend_compute_resid (GMT, G, T, R);
 
 		if (Ctrl->N.robust) {
-			chisq = compute_chisq (GMT, R, W, scale);
+			chisq = grdtrend_compute_chisq (GMT, R, W, scale);
 			iterations = 1;
 			sprintf (format, "grdtrend: Robust iteration %%d:  Old Chi Squared: %s  New Chi Squared: %s\n", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
 			do {
 				old_chisq = chisq;
 				gmt_M_memcpy (old, gtd, Ctrl->N.value, double);
-				scale = compute_robust_weight (GMT, R, W);
-				load_gtg_and_gtd (GMT, G, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, W, weighted);
+				scale = grdtrend_compute_robust_weight (GMT, R, W);
+				grdtrend_load_gtg_and_gtd (GMT, G, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, W, weighted);
 				ierror = gmt_gauss (GMT, gtg, gtd, Ctrl->N.value, Ctrl->N.value, true);
 				if (ierror) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Gauss returns error code %d\n", ierror);
-					return (GMT_RUNTIME_ERROR);
+					GMT_Report (API, GMT_MSG_ERROR, "Gauss returns error code %d\n", ierror);
+					error = GMT_RUNTIME_ERROR;
+					goto END;
 				}
-				compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
-				compute_resid (GMT, G, T, R);
-				chisq = compute_chisq (GMT, R, W, scale);
-				GMT_Report (API, GMT_MSG_VERBOSE, format, iterations, old_chisq, chisq);
+				grdtrend_compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
+				grdtrend_compute_resid (GMT, G, T, R);
+				chisq = grdtrend_compute_chisq (GMT, R, W, scale);
+				GMT_Report (API, GMT_MSG_INFORMATION, format, iterations, old_chisq, chisq);
 				iterations++;
 			} while (old_chisq / chisq > 1.0001);
 
 			/* Get here when new model not significantly better; use old one */
 
 			gmt_M_memcpy (gtd, old, Ctrl->N.value, double);
-			compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
-			compute_resid (GMT, G, T, R);
+			grdtrend_compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
+			grdtrend_compute_resid (GMT, G, T, R);
 		}
 	}
 
@@ -634,43 +660,46 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 
 	/* Get here when ready to do output */
 
-	if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) write_model_parameters (GMT, gtd, Ctrl->N.value);
+	if (gmt_M_is_verbose (GMT, GMT_MSG_WARNING)) grdtrend_write_model_parameters (GMT, gtd, Ctrl->N.value);
 	if (Ctrl->T.file) {
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "trend surface", T)) Return (API->error);
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, T)) Return (API->error);
-		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_ALL, NULL, Ctrl->T.file, T) != GMT_NOERROR) {
-			Return (API->error);
+		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->T.file, T) != GMT_NOERROR) {
+			error = API->error;
+			goto END;
 		}
 	}
 	else if (GMT_Destroy_Data (API, &T) != GMT_NOERROR) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Failed to free T\n");
+		GMT_Report (API, GMT_MSG_ERROR, "Failed to free T\n");
 	}
 	if (Ctrl->D.file) {
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "trend residuals", R)) Return (API->error);
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, R)) Return (API->error);
-		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_ALL, NULL, Ctrl->D.file, R) != GMT_NOERROR) {
-			Return (API->error);
+		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->D.file, R) != GMT_NOERROR) {
+			error = API->error;
+			goto END;
 		}
 	}
 	else if (Ctrl->D.active || Ctrl->N.robust) {
 		if (GMT_Destroy_Data (API, &R) != GMT_NOERROR) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Failed to free R\n");
+			GMT_Report (API, GMT_MSG_ERROR, "Failed to free R\n");
 		}
 	}
 	if (Ctrl->W.file && Ctrl->N.robust) {
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "trend weights", W)) Return (API->error);
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, W)) Return (API->error);
-		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_ALL, NULL, Ctrl->W.file, W) != GMT_NOERROR) {
-			Return (API->error);
+		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->W.file, W) != GMT_NOERROR) {
+			error = API->error;
+			goto END;
 		}
 	}
 	else if (set_ones && GMT_Destroy_Data (API, &W) != GMT_NOERROR) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Failed to free W\n");
+		GMT_Report (API, GMT_MSG_ERROR, "Failed to free W\n");
 	}
 
 	/* That's all, folks!  */
 
-
+END:
 	gmt_M_free (GMT, pstuff);
 	gmt_M_free (GMT, gtd);
 	gmt_M_free (GMT, gtg);
@@ -678,7 +707,5 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 	gmt_M_free (GMT, yval);
 	gmt_M_free (GMT, xval);
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Done!\n");
-
-	Return (GMT_NOERROR);
+	Return (error);
 }
