@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77info.c 12822 2014-01-31 23:39:56Z remko $
  *
- *    Copyright (c) 2004-2014 by P. Wessel
+ *    Copyright (c) 2004-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -20,16 +19,18 @@
  *
  *
  */
- 
-#define THIS_MODULE_NAME	"mgd77info"
-#define THIS_MODULE_LIB		"mgd77"
-#define THIS_MODULE_PURPOSE	"Extract information about MGD77 files"
 
 #include "gmt_dev.h"
 #include "mgd77.h"
 #include "mgd77_codes.h"
 
-#define GMT_PROG_OPTIONS "-V"
+#define THIS_MODULE_CLASSIC_NAME	"mgd77info"
+#define THIS_MODULE_MODERN_NAME	"mgd77info"
+#define THIS_MODULE_LIB		"mgd77"
+#define THIS_MODULE_PURPOSE	"Extract information about MGD77 files"
+#define THIS_MODULE_KEYS	""
+#define THIS_MODULE_NEEDS	""
+#define THIS_MODULE_OPTIONS "-V"
 
 #define FORMATTED_HEADER	1
 #define RAW_HEADER		2
@@ -38,60 +39,59 @@
 
 struct MGD77INFO_CTRL {	/* All control options for this program (except common args) */
 	/* active is true if the option has been activated */
-	struct C {	/* -C */
+	struct MGD77INFO_C {	/* -C */
 		bool active;
 		unsigned int mode;
 	} C;
-	struct E {	/* -E */
+	struct MGD77INFO_E {	/* -E */
 		bool active;
 		unsigned int mode;
 	} E;
-	struct I {	/* -I */
+	struct MGD77INFO_I {	/* -I */
 		bool active;
 		unsigned int n;
 		char code[3];
 	} I;
-	struct L {	/* -L */
+	struct MGD77INFO_L {	/* -L */
 		bool active;
 		unsigned int mode;
 	} L;
-	struct M {	/* -M */
+	struct MGD77INFO_M {	/* -M */
 		bool active;
 		unsigned int mode;
 		unsigned int flag;
 	} M;
 };
 
-void *New_mgd77info_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+static void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct MGD77INFO_CTRL *C = NULL;
-	
-	C = GMT_memory (GMT, NULL, 1, struct MGD77INFO_CTRL);
-	
+
+	C = gmt_M_memory (GMT, NULL, 1, struct MGD77INFO_CTRL);
+
 	/* Initialize values whose defaults are not 0/false/NULL */
-	
-	C->C.mode = 3;	
+
+	C->C.mode = 3;
 	C->L.mode = 1;
 	return (C);
 }
 
-void Free_mgd77info_Ctrl (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *C) {	/* Deallocate control structure */
+static void Free_Ctrl (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
-	GMT_free (GMT, C);	
+	gmt_M_free (GMT, C);
 }
 
-int GMT_mgd77info_usage (struct GMTAPI_CTRL *API, int level)
-{
+static int usage (struct GMTAPI_CTRL *API, int level) {
 	struct MGD77_CONTROL M;
 
-	GMT_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: mgd77info <cruise(s)> [-C[m|e]] [-E[m|e]] [-I<code>] [-Mf[<item>]|r|e|h] [-L[v]]\n\t[%s]\n\n", GMT_V_OPT);
-        
-	if (level == GMT_SYNOPSIS) return (EXIT_FAILURE);
-             
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <cruise(s)> [-C[m|e]] [-E[m|e]] [-I<code>] [-Mf[<item>]|r|e|h] [-L[v]]\n\t[%s] [%s]\n\n", name, GMT_V_OPT, GMT_PAR_OPT);
+
+	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
+
 	MGD77_Init (API->GMT, &M);		/* Initialize MGD77 Machinery */
 	MGD77_Cruise_Explain (API->GMT);
-	GMT_Message (API, GMT_TIME_NONE, "\tOPTIONS:\n\n");
+	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-C List abbreviations of all columns present for each cruise.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append m for listing just the MGD77 columns present.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append e for listing just any extra columns present.\n");
@@ -110,21 +110,20 @@ int GMT_mgd77info_usage (struct GMTAPI_CTRL *API, int level)
 	GMT_Message (API, GMT_TIME_NONE, "\t-L List all the institutions and their 2-character GEODAS codes only.  Append v to also\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   display the vessels and their 4-character codes for each institution.\n");
 	GMT_Option (API, "V,.");
-	
+
 	MGD77_end (API->GMT, &M);	/* Close machinery */
 
-	return (EXIT_FAILURE);
+	return (GMT_MODULE_USAGE);
 }
 
-int GMT_mgd77info_parse (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *Ctrl, struct GMT_OPTION *options, struct MGD77_CONTROL *M)
-{
+static int parse (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *Ctrl, struct GMT_OPTION *options, struct MGD77_CONTROL *M) {
 	/* This parses the options provided to mgd77info and sets parameters in CTRL.
 	 * Any GMT common options will override values set previously by other commands.
 	 * It also replaces any file names specified as input or output with the data ID
 	 * returned when registering these sources/destinations with the API.
 	 */
 
-	unsigned int n_errors = 0;
+	unsigned int n_errors = 0, n_opts = 0;
 	int sval;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
@@ -173,7 +172,7 @@ int GMT_mgd77info_parse (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *Ctrl, stru
 					Ctrl->M.mode = HIST_HEADER;
 				}
 				else {
-					GMT_Report (API, GMT_MSG_NORMAL, "Option -M Bad modifier (%c). Use -Mf|r|e|h!\n", opt->arg[0]);
+					GMT_Report (API, GMT_MSG_ERROR, "Option -M Bad modifier (%c). Use -Mf|r|e|h!\n", opt->arg[0]);
 					n_errors++;
 				}
 				break;
@@ -181,15 +180,15 @@ int GMT_mgd77info_parse (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *Ctrl, stru
 			case 'I':
 				Ctrl->I.active = true;
 				if (Ctrl->I.n < 3) {
-					if (strchr ("act", (int)opt->arg[0]))
+					if (strchr ("acmt", (int)opt->arg[0]))
 						Ctrl->I.code[Ctrl->I.n++] = opt->arg[0];
 					else {
-						GMT_Report (API, GMT_MSG_NORMAL, "Option -I Bad modifier (%c). Use -Ia|c|t!\n", opt->arg[0]);
+						GMT_Report (API, GMT_MSG_ERROR, "Option -I Bad modifier (%c). Use -Ia|c|m|t!\n", opt->arg[0]);
 						n_errors++;
 					}
 				}
 				else {
-					GMT_Report (API, GMT_MSG_NORMAL, "Option -I: Can only be applied 0-2 times\n");
+					GMT_Report (API, GMT_MSG_ERROR, "Option -I: Can only be applied 0-2 times\n");
 					n_errors++;
 				}
 				break;
@@ -208,7 +207,7 @@ int GMT_mgd77info_parse (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *Ctrl, stru
 						Ctrl->E.mode = 3;
 						break;
 					default:
-						GMT_Report (API, GMT_MSG_NORMAL, "Option -E Bad modifier (%c). Use -E[e|m]!\n", opt->arg[0]);
+						GMT_Report (API, GMT_MSG_ERROR, "Option -E Bad modifier (%c). Use -E[e|m]!\n", opt->arg[0]);
 						n_errors++;
 				}
 				Ctrl->E.active = true;
@@ -226,83 +225,86 @@ int GMT_mgd77info_parse (struct GMT_CTRL *GMT, struct MGD77INFO_CTRL *Ctrl, stru
 					case '\0':
 						break;
 					default:
-						GMT_Report (API, GMT_MSG_NORMAL, "Option -L Bad modifier (%c). Use -L[a|v]!\n", opt->arg[0]);
+						GMT_Report (API, GMT_MSG_ERROR, "Option -L Bad modifier (%c). Use -L[a|v]!\n", opt->arg[0]);
 						n_errors++;
 				}
 				break;
 			default:	/* Report bad options */
-				n_errors += GMT_default_error (GMT, opt->option);
+				n_errors += gmt_default_error (GMT, opt->option);
 				break;
 		}
 	}
+	if (Ctrl->M.mode == RAW_HEADER) n_opts++;
+	if (Ctrl->M.mode == E77_HEADER) n_opts++;
+	if (Ctrl->M.mode == HIST_HEADER) n_opts++;
+	if (Ctrl->M.mode == FORMATTED_HEADER) n_opts++;
+	if (Ctrl->E.active) n_opts++;
+	if (Ctrl->C.active) n_opts++;
+	if (Ctrl->L.active) n_opts++;
+	n_errors += gmt_M_check_condition (GMT, n_opts != 1, "Specify one of -C, -E, -L, or -M\n");
 
-	n_errors += GMT_check_condition (GMT, !((Ctrl->M.mode == RAW_HEADER) + (Ctrl->M.mode == E77_HEADER) + (Ctrl->M.mode == HIST_HEADER) \
-		+ Ctrl->E.active + Ctrl->C.active + (Ctrl->M.mode == FORMATTED_HEADER) + Ctrl->L.active ) == 1, "Syntax error: Specify one of -C, -E, -L, or -M\n");
-
-	return (n_errors ? GMT_PARSE_ERROR : GMT_OK);
+	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-#define bailout(code) {GMT_Free_Options (mode); return (code);}
-#define Return(code) {Free_mgd77info_Ctrl (GMT, Ctrl); GMT_end_module (GMT, GMT_cpy); bailout (code);}
+#define bailout(code) {gmt_M_free_options (mode); return (code);}
+#define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
+
+EXTERN_MSC int GMT_mgd77info (void *V_API, int mode, void *args) {
+	int i, id, id_col, t_col, x_col, y_col, error = 0, argno, n_paths;
 
-int GMT_mgd77info (void *V_API, int mode, void *args)
-{
-	int i, id, id_col, t_col, x_col, y_col, error = 0;
-	
 	int64_t rata_die;
 	size_t length;
-	
-	uint64_t rec, argno, n_paths, counter[MGD77_MAX_COLS];
+
+	uint64_t rec, counter[MGD77_MAX_COLS];
 	unsigned int saved_range, quad_no, n_quad, use, k;
 	bool first = true, read_file, quad[4] = {false, false, false, false};
-	
+
 	double this_dist, this_lon, this_lat, last_lon, last_lat, dx, dy, dlon, ds, lon_w;
 	double xmin, xmax, xmin1, xmin2, xmax1, xmax2, ymin, ymax, this_time, tmin, tmax;
 	double *dvalue[MGD77_MAX_COLS];
-	
+
 	char *tvalue[MGD77_MAX_COLS], **list = NULL;
-		
+
 	struct MGD77_CONTROL M, Out;
 	struct MGD77_DATASET *D = NULL;
 	struct MGD77INFO_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
-	struct GMTAPI_CTRL *API = GMT_get_API_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
+	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 
 	/*----------------------- Standard module initialization and parsing ----------------------*/
 
 	if (API == NULL) return (GMT_NOT_A_SESSION);
-	if (mode == GMT_MODULE_PURPOSE) return (GMT_mgd77info_usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
+	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (GMT_mgd77info_usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (GMT_mgd77info_usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	GMT = GMT_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
-	Ctrl = New_mgd77info_Ctrl (GMT);	/* Allocate and initialize a new control structure */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
+	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	MGD77_Init (GMT, &M);		/* Initialize MGD77 Machinery */
-	if ((error = GMT_mgd77info_parse (GMT, Ctrl, options, &M))) Return (error);
+	if ((error = parse (GMT, Ctrl, options, &M)) != 0) Return (error);
 
 	/*---------------------------- This is the mgd77info main code ----------------------------*/
 
 	if (Ctrl->M.flag == 1) {
 		MGD77_List_Header_Items (GMT, &M);
 		MGD77_end (GMT, &M);
-		Return (GMT_OK);
+		Return (GMT_NOERROR);
 	}
 
 	/* Initialize MGD77 output order and other parameters*/
-	
+
 	MGD77_Init (GMT, &Out);		/* Initialize output MGD77 Machinery */
 	Out.fp = GMT->session.std[GMT_OUT];
 
 	if (Ctrl->I.active) MGD77_Process_Ignore (GMT, 'I', Ctrl->I.code);
 
 	/* Check that the options selected are mutually consistent */
-	
+
 	if (Ctrl->L.active) {	/* Just display the list and exit */
 		(Ctrl->L.mode == 2) ? printf ("CODE\tINSTITUTION/VESSEL\n") : printf ("CODE\tINSTITUTION\n");
 		for (id = i = 0; id < MGD77_N_AGENCIES; id++) {
@@ -311,42 +313,46 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 				printf ("%s\t-> %s\n", MGD77_vessel[i].code, MGD77_vessel[i].name);
 			}
 		}
-		Return (GMT_OK);
+		Return (GMT_NOERROR);
 	}
 
 	n_paths = MGD77_Path_Expand (GMT, &M, options, &list);	/* Get list of requested IDs */
-	
-	if (n_paths == 0) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: No cruises given\n");
-		Return (EXIT_FAILURE);
+
+	if (n_paths <= 0) {
+		GMT_Report (API, GMT_MSG_ERROR, "No cruises given\n");
+		Return (GMT_NO_INPUT);
 	}
-	
+
 	read_file = (Ctrl->E.active || (Ctrl->M.mode == RAW_HEADER));
-	
+
 	saved_range = GMT->current.io.geo.range;	/* We may have to reset thisso keep a copy */
-	GMT_set_geographic (GMT, GMT_OUT);	/* Output lon/lat */
-	GMT->current.io.col_type[GMT_OUT][GMT_Z] = M.time_format;	
+	gmt_set_geographic (GMT, GMT_OUT);	/* Output lon/lat */
+	gmt_set_column_type (GMT, GMT_OUT, GMT_Z, M.time_format);
 	if (Ctrl->E.active) fprintf (GMT->session.std[GMT_OUT], "#Cruise %sID      %sWest    %sEast    %sSouth   %sNorth   %sStartTime%s%sEndTime%s%s%sDist%snRec",
 		GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator,
 		GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator);
 
 	use = (M.original || M.format != MGD77_FORMAT_CDF) ? MGD77_ORIG : MGD77_REVISED;
-	
+
 	for (argno = 0; argno < n_paths; argno++) {		/* Process each ID */
-	
+
 		if (MGD77_Open_File (GMT, list[argno], &M, MGD77_READ_MODE)) continue;
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Now processing cruise %s\n", list[argno]);
-		
+		GMT_Report (API, GMT_MSG_INFORMATION, "Now processing cruise %s\n", list[argno]);
+
 		D = MGD77_Create_Dataset (GMT);
-		
+
 		if (read_file && MGD77_Read_File (GMT, list[argno], &M, D)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Error reading header & data for cruise %s\n", list[argno]);
-			Return (EXIT_FAILURE);
+			GMT_Report (API, GMT_MSG_ERROR, "Failure while reading header & data for cruise %s\n", list[argno]);
+			MGD77_Close_File (GMT, &M);
+			MGD77_Free_Dataset (GMT, &D);
+			Return (GMT_DATA_READ_ERROR);
 		}
 		if (!read_file && MGD77_Read_Header_Record (GMT, list[argno], &M, &D->H)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Error reading header sequence for cruise %s\n", list[argno]);
-			Return (EXIT_FAILURE);
+			GMT_Report (API, GMT_MSG_ERROR, "Failure while reading header sequence for cruise %s\n", list[argno]);
+			MGD77_Close_File (GMT, &M);
+			MGD77_Free_Dataset (GMT, &D);
+			Return (GMT_DATA_READ_ERROR);
 		}
 
 		if (Ctrl->M.mode == HIST_HEADER) {	/* Dump of MGD77+ history */
@@ -365,7 +371,7 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 			continue;
 		}
 		if (Ctrl->M.mode == FORMATTED_HEADER) {	/* Dump of header items, one per line */
-			MGD77_Dump_Header_Params (GMT, &M, D->H.mgd77[use]);	
+			MGD77_Dump_Header_Params (GMT, &M, D->H.mgd77[use]);
 			MGD77_Close_File (GMT, &M);
 			MGD77_Free_Dataset (GMT, &D);
 			continue;
@@ -374,7 +380,7 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 		x_col = MGD77_Get_Column (GMT, "lon", &M);
 		y_col = MGD77_Get_Column (GMT, "lat", &M);
 		id_col = MGD77_Get_Column (GMT, "id", &M);
-		
+
 		if (first && Ctrl->E.active) {	/* Output all column headers */
 			for (i = k = 0; k < M.n_out_columns; i++, k++) {
 				if (i == id_col || i == t_col || i == x_col || i == y_col) continue;
@@ -382,7 +388,7 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 			}
 			fprintf (GMT->session.std[GMT_OUT],"\n");
 		}
-		
+
 		if (Ctrl->C.active) {	/* Just list names and info for any extra columns */
 			for (i = k = 0, first = true; k < M.n_out_columns; i++, k++) {
 				if (i == id_col || i == t_col || i == x_col || i == y_col) continue;
@@ -398,7 +404,7 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 			MGD77_Free_Dataset (GMT, &D);
 			continue;
 		}
-			
+
 		if (Ctrl->M.mode == RAW_HEADER) {	/* Write entire MGD77 header */
 			fprintf (GMT->session.std[GMT_OUT], "-------------------------------");
 			fprintf (GMT->session.std[GMT_OUT], " Cruise: %8s ", M.NGDC_id);
@@ -419,28 +425,28 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 			}
 			fprintf (GMT->session.std[GMT_OUT], "\n");
 		}
-		
+
 		tmin = tmax = GMT->session.d_NaN;
 		this_dist = this_lon = this_lat = ds = this_time = 0.0;
 		xmin1 = xmin2 = 360.0;
 		xmax1 = xmax2 = -360.0;
 		ymin = 180.0;
 		ymax = -180.0;
-		GMT_memset (quad, 4, bool);	/* Set all to false */
-		GMT_memset (counter, MGD77_MAX_COLS, uint64_t);
-	
+		gmt_M_memset (quad, 4, bool);	/* Set all to false */
+		gmt_M_memset (counter, MGD77_MAX_COLS, uint64_t);
+
 		for (i = 0; i < MGD77_MAX_COLS; i++) {
 			dvalue[i] = D->values[i];
 			tvalue[i] = D->values[i];
 		}
-		
+
 		/* Start processing data */
-	
+
 		for (rec = 0; rec < D->H.n_records; rec++) {		/* While able to read a data record */
-		
+
 			/* Get min and max time */
-			if (t_col >= 0 && !GMT_is_dnan(dvalue[t_col][rec])) {
-				if (GMT_is_dnan(tmin) && GMT_is_dnan(tmax)) tmin = tmax = dvalue[t_col][rec];
+			if (t_col >= 0 && !gmt_M_is_dnan(dvalue[t_col][rec])) {
+				if (gmt_M_is_dnan(tmin) && gmt_M_is_dnan(tmax)) tmin = tmax = dvalue[t_col][rec];
 				this_time = dvalue[t_col][rec];
 				tmin = MIN (this_time, tmin);
 				tmax = MAX (this_time, tmax);
@@ -461,7 +467,7 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 			xmin2 = MIN (this_lon, xmin2);
 			xmax2 = MAX (this_lon, xmax2);
 			if (rec > 0) {	/* Need a previous point to calculate distance, speed, and heading */
-				GMT_set_delta_lon (last_lon, this_lon, dlon);
+				gmt_M_set_delta_lon (last_lon, this_lon, dlon);
 				dx = dlon * cosd (0.5 * (this_lat + last_lat));
 				dy = this_lat - last_lat;
 				ds = GMT->current.proj.DIST_KM_PR_DEG * hypot (dx, dy);
@@ -469,16 +475,16 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 			}
 			ymin = MIN (this_lat, ymin);
 			ymax = MAX (this_lat, ymax);
-			
+
 			/* Count the number of non-NaN observations */
-			
+
 			for (i = k = 1; k < M.n_out_columns; i++, k++) {
 				if (i == id_col || i == t_col || i == x_col || i == y_col) continue;
-				if ((length = D->H.info[M.order[k].set].col[M.order[k].item].text)) {
+				if ((length = D->H.info[M.order[k].set].col[M.order[k].item].text) != 0) {
 					if (strncmp (&tvalue[k][rec*length], ALL_NINES, length)) counter[k]++;
 				}
 				else
-					if (!GMT_is_dnan (dvalue[k][rec])) counter[k]++;
+					if (!gmt_M_is_dnan (dvalue[k][rec])) counter[k]++;
 			}
 		}
 
@@ -513,9 +519,9 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 		if (xmin > xmax) xmin -= 360.0;
 		if (xmin < 0.0 && xmax < 0.0) xmin += 360.0, xmax += 360.0;
 
-		if (GMT_is_dnan(tmin) || GMT_is_dnan(tmax)) {
+		if (gmt_M_is_dnan(tmin) || gmt_M_is_dnan(tmax)) {
 			int yy[2], mm[2], dd[2];
-			GMT_Report (API, GMT_MSG_VERBOSE, "warning: cruise %s no time records.\n", M.NGDC_id);
+			GMT_Report (API, GMT_MSG_WARNING, "Cruise %s no time records.\n", M.NGDC_id);
 			yy[0] = (!D->H.mgd77[use]->Survey_Departure_Year[0] || !strncmp (D->H.mgd77[use]->Survey_Departure_Year, ALL_BLANKS, 4U)) ? 0 : atoi (D->H.mgd77[use]->Survey_Departure_Year);
 			yy[1] = (!D->H.mgd77[use]->Survey_Arrival_Year[0] || !strncmp (D->H.mgd77[use]->Survey_Arrival_Year, ALL_BLANKS, 4U)) ? 0 : atoi (D->H.mgd77[use]->Survey_Arrival_Year);
 			mm[0] = (!D->H.mgd77[use]->Survey_Departure_Month[0] || !strncmp (D->H.mgd77[use]->Survey_Departure_Month, ALL_BLANKS, 2U)) ? 1 : atoi (D->H.mgd77[use]->Survey_Departure_Month);
@@ -523,21 +529,21 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 			dd[0] = (!D->H.mgd77[use]->Survey_Departure_Day[0] || !strncmp (D->H.mgd77[use]->Survey_Departure_Day, ALL_BLANKS, 2U)) ? 1 : atoi (D->H.mgd77[use]->Survey_Departure_Day);
 			dd[1] = (!D->H.mgd77[use]->Survey_Arrival_Day[0] || !strncmp (D->H.mgd77[use]->Survey_Arrival_Day, ALL_BLANKS, 2U)) ? 1 : atoi (D->H.mgd77[use]->Survey_Arrival_Day);
 			if (! (yy[0] == 0 && yy[1] == 0)) {	/* With year we can do something */
-				rata_die = GMT_rd_from_gymd (GMT, yy[0], mm[0], dd[0]);
-				tmin = GMT_rdc2dt (GMT, rata_die, 0.0);
-				rata_die = GMT_rd_from_gymd (GMT, yy[1], mm[1], dd[1]);
-				tmax = GMT_rdc2dt (GMT, rata_die, 0.0);
+				rata_die = gmt_rd_from_gymd (GMT, yy[0], mm[0], dd[0]);
+				tmin = gmt_rdc2dt (GMT, rata_die, 0.0);
+				rata_die = gmt_rd_from_gymd (GMT, yy[1], mm[1], dd[1]);
+				tmax = gmt_rdc2dt (GMT, rata_die, 0.0);
 			}
-		}			
+		}
 		if (Ctrl->E.active) {
 			fprintf (GMT->session.std[GMT_OUT],"%8s%s%8s%s", M.NGDC_id, GMT->current.setting.io_col_separator, D->H.mgd77[use]->Survey_Identifier, GMT->current.setting.io_col_separator);
-			GMT_ascii_output_col (GMT, GMT->session.std[GMT_OUT], xmin, GMT_X);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
-			GMT_ascii_output_col (GMT, GMT->session.std[GMT_OUT], xmax, GMT_X);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
-			GMT_ascii_output_col (GMT, GMT->session.std[GMT_OUT], ymin, GMT_Y);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
-			GMT_ascii_output_col (GMT, GMT->session.std[GMT_OUT], ymax, GMT_Y);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
-			if (!GMT_is_dnan(tmin) && !GMT_is_dnan(tmax)) {
-				GMT_ascii_output_col (GMT, GMT->session.std[GMT_OUT], tmin, GMT_Z);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
-				GMT_ascii_output_col (GMT, GMT->session.std[GMT_OUT], tmax, GMT_Z);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);						
+			gmt_ascii_output_col (GMT, GMT->session.std[GMT_OUT], xmin, GMT_X);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
+			gmt_ascii_output_col (GMT, GMT->session.std[GMT_OUT], xmax, GMT_X);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
+			gmt_ascii_output_col (GMT, GMT->session.std[GMT_OUT], ymin, GMT_Y);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
+			gmt_ascii_output_col (GMT, GMT->session.std[GMT_OUT], ymax, GMT_Y);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
+			if (!gmt_M_is_dnan(tmin) && !gmt_M_is_dnan(tmax)) {
+				gmt_ascii_output_col (GMT, GMT->session.std[GMT_OUT], tmin, GMT_Z);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
+				gmt_ascii_output_col (GMT, GMT->session.std[GMT_OUT], tmax, GMT_Z);	fprintf (GMT->session.std[GMT_OUT], "%s", GMT->current.setting.io_col_separator);
 			} else {
 				fprintf (GMT->session.std[GMT_OUT], "%4s-%2s-%2s%s%4s-%2s-%2s%s",
 				D->H.mgd77[use]->Survey_Departure_Year, D->H.mgd77[use]->Survey_Departure_Month, D->H.mgd77[use]->Survey_Departure_Day, GMT->current.setting.io_col_separator,
@@ -553,10 +559,10 @@ int GMT_mgd77info (void *V_API, int mode, void *args)
 		}
 		MGD77_Free_Dataset (GMT, &D);
 	}
-		
-	MGD77_Path_Free (GMT, n_paths, list);
+
+	MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 	MGD77_end (GMT, &M);
 	MGD77_end (GMT, &Out);
 
-	Return (GMT_OK);
+	Return (GMT_NOERROR);
 }
