@@ -1,7 +1,6 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys_report.c 17503 2017-01-30 23:14:43Z pwessel $
  *
- *      Copyright (c) 1999-2017 by P. Wessel
+ *      Copyright (c) 1999-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *      Contact info: gmt.soest.hawaii.edu
+ *      Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* x2sys_report will read the crossover data base and report on the statistics
  * of the crossovers for each track and overall.
@@ -25,51 +24,54 @@
  *
  */
 
-#define THIS_MODULE_NAME	"x2sys_report"
-#define THIS_MODULE_LIB		"x2sys"
-#define THIS_MODULE_PURPOSE	"Report statistics from crossover data base"
-#define THIS_MODULE_KEYS	">T}"
-
+#include "gmt_dev.h"
+#include "mgd77/mgd77.h"
 #include "x2sys.h"
 
-#define GMT_PROG_OPTIONS "->RV"
+#define THIS_MODULE_CLASSIC_NAME	"x2sys_report"
+#define THIS_MODULE_MODERN_NAME	"x2sys_report"
+#define THIS_MODULE_LIB		"x2sys"
+#define THIS_MODULE_PURPOSE	"Report statistics from crossover data base"
+#define THIS_MODULE_KEYS	">D}"
+#define THIS_MODULE_NEEDS	""
+#define THIS_MODULE_OPTIONS "->RV"
 
 #define XREPORT_EXTERNAL	1
 #define XREPORT_INTERNAL	2
 
 struct X2SYS_REPORT_CTRL {
-	struct In {
+	struct X2SYS_REPORT_In {
 		bool active;
 		char *file;
 	} In;
-	struct A {	/* -A */
+	struct X2SYS_REPORT_A {	/* -A */
 		bool active;
 	} A;
-	struct C {	/* -C */
+	struct X2SYS_REPORT_C {	/* -C */
 		bool active;
 		char *col;
 	} C;
-	struct I {	/* -I */
+	struct X2SYS_REPORT_I {	/* -I */
 		bool active;
 		char *file;
 	} I;
-	struct L {	/* -L */
+	struct X2SYS_REPORT_L {	/* -L */
 		bool active;
 		char *file;
 	} L;
-	struct N {	/* -N */
+	struct X2SYS_REPORT_N {	/* -N */
 		bool active;
 		uint64_t min;
 	} N;
-	struct Q {	/* -Q */
+	struct X2SYS_REPORT_Q {	/* -Q */
 		bool active;
 		int mode;
 	} Q;
-	struct S {	/* -S */
+	struct X2SYS_REPORT_S {	/* -S */
 		bool active;
 		char *file;
 	} S;
-	struct T {	/* -T */
+	struct X2SYS_REPORT_T {	/* -T */
 		bool active;
 		char *TAG;
 	} T;
@@ -93,7 +95,7 @@ struct COE_ADJLIST {	/* Array with the growing arrays of COE_ADJUST per track */
 	size_t n_alloc;
 };
 
-GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+static void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct X2SYS_REPORT_CTRL *C;
 
 	C = gmt_M_memory (GMT, NULL, 1, struct X2SYS_REPORT_CTRL);
@@ -101,7 +103,7 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	return (C);
 }
 
-GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct X2SYS_REPORT_CTRL *C) {	/* Deallocate control structure */
+static void Free_Ctrl (struct GMT_CTRL *GMT, struct X2SYS_REPORT_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
 	gmt_M_str_free (C->In.file);
 	gmt_M_str_free (C->C.col);
@@ -112,11 +114,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct X2SYS_REPORT_CTRL *C) {	/
 	gmt_M_free (GMT, C);
 }
 
-GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+static int usage (struct GMTAPI_CTRL *API, int level) {
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: x2sys_report -C<column> -T<TAG> [<COEdbase>] [-A] [-I<ignorelist>] [-L[<corrtable.txt>]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-N<nx_min>] [-Qe|i] [-S<track>] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s -C<column> -T<TAG> [<COEdbase>] [-A] [-I<ignorelist>] [-L[<corrtable.txt>]]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-N<nx_min>] [-Qe|i] [-S<track>] [%s] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -132,14 +134,14 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Output results for tracks with more than <nx_min> crossovers only [0, i.e., report all tracks].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Append e or i for external or internal crossovers [Default is external].\n");
 	GMT_Option (API, "R");
-	GMT_Message (API, GMT_TIME_NONE, "\t   [Default region is the entire data domain].\n");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   [Default region is the entire data domain].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Return only crossovers involving this track [Use all tracks].\n");
 	GMT_Option (API, "V,.");
-	
+
 	return (GMT_MODULE_USAGE);
 }
 
-GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_REPORT_CTRL *Ctrl, struct GMT_OPTION *options) {
+static int parse (struct GMT_CTRL *GMT, struct X2SYS_REPORT_CTRL *Ctrl, struct GMT_OPTION *options) {
 
 	/* This parses the options provided to grdcut and sets parameters in CTRL.
 	 * Any GMT common options will override values set previously by other commands.
@@ -162,7 +164,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_REPORT_CTRL *Ctrl, struc
 				break;
 
 			/* Processes program-specific parameters */
-			
+
 			case 'A':
 				Ctrl->A.active = true;
 				break;
@@ -203,14 +205,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_REPORT_CTRL *Ctrl, struc
 		}
 	}
 
-	n_errors += gmt_M_check_condition (GMT, !Ctrl->T.active || !Ctrl->T.TAG, "Syntax error: -T must be used to set the TAG\n");
-	n_errors += gmt_M_check_condition (GMT, !Ctrl->C.active || !Ctrl->C.col, "Syntax error: Must use -C to specify observation of interest\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->Q.mode == 3, "Syntax error: Only one of -Qe -Qi can be specified!\n");
+	n_errors += gmt_M_check_condition (GMT, !Ctrl->T.active || !Ctrl->T.TAG, "Option -T must be used to set the TAG\n");
+	n_errors += gmt_M_check_condition (GMT, !Ctrl->C.active || !Ctrl->C.col, "Must use -C to specify observation of interest\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->Q.mode == 3, "Only one of -Qe -Qi can be specified!\n");
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-GMT_LOCAL int comp_structs (const void *point_1, const void *point_2) { /* Sort ADJ structure on distance */
+GMT_LOCAL int x2sysreport_comp_structs (const void *point_1, const void *point_2) { /* Sort ADJ structure on distance */
         if ( ((struct COE_ADJUST *)point_1)->d < ((struct COE_ADJUST *)point_2)->d)
                 return(-1);
         else if ( ((struct COE_ADJUST *)point_1)->d > ((struct COE_ADJUST *)point_2)->d)
@@ -222,7 +224,7 @@ GMT_LOCAL int comp_structs (const void *point_1, const void *point_2) { /* Sort
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-int GMT_x2sys_report (void *V_API, int mode, void *args) {
+EXTERN_MSC int GMT_x2sys_report (void *V_API, int mode, void *args) {
 	char **trk_name = NULL, *c = NULL, fmt[GMT_BUFSIZ] = {""}, record[GMT_BUFSIZ] = {""}, word[GMT_BUFSIZ] = {""};
 	struct X2SYS_INFO *s = NULL;
 	struct X2SYS_BIX B;
@@ -232,11 +234,14 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 	int error = 0;
 	bool internal = false;	/* false if only external xovers are needed */
 	bool external = true;	/* false if only internal xovers are needed */
+	int64_t value;
 	uint64_t i, k, n, n_use, n_tracks;
 	uint64_t p, np, nx, Tnx = 0;
+	size_t len;
 	unsigned int coe_kind;
 	double sum, sum2, sum_w, Tsum, Tsum2, COE, sign, scale, corr[2] = {0.0, 0.0};
 	double Tmean, Tstdev, Trms;
+	struct GMT_RECORD *Out = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct X2SYS_REPORT_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
@@ -249,35 +254,36 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
-	
+
 	/*---------------------------- This is the x2sys_report main code ----------------------------*/
 
 	c = GMT->current.setting.io_col_separator;
-	
+
 	/* Initialize system via the tag */
-	
-	x2sys_err_fail (GMT, x2sys_set_system (GMT, Ctrl->T.TAG, &s, &B, &GMT->current.io), Ctrl->T.TAG);
+
+	if (x2sys_err_fail (GMT, x2sys_set_system (GMT, Ctrl->T.TAG, &s, &B, &GMT->current.io), Ctrl->T.TAG))
+		Return (GMT_RUNTIME_ERROR);
 
 	/* Verify that the chosen column is known to the system */
-	
-	if (Ctrl->C.col) x2sys_err_fail (GMT, x2sys_pick_fields (GMT, Ctrl->C.col, s), "-C");
+
+	if (Ctrl->C.col && x2sys_err_fail (GMT, x2sys_pick_fields (GMT, Ctrl->C.col, s), "-C"))
+		Return (GMT_RUNTIME_ERROR);
 	if (s->n_out_columns != 1) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: -C must specify a single column name\n");
+		GMT_Report (API, GMT_MSG_ERROR, "-C must specify a single column name\n");
 		x2sys_end (GMT, s);
 		Return (GMT_RUNTIME_ERROR);
 	}
-	
+
 	/* Select internal, external, or both */
-	
+
 	if (Ctrl->Q.active) {	/* Follow user options */
 		internal = (Ctrl->Q.mode == XREPORT_EXTERNAL) ? false : true;
 		external = (Ctrl->Q.mode == XREPORT_INTERNAL) ? false : true;
@@ -286,27 +292,33 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 	if (internal) coe_kind |= 1;
 	if (external) coe_kind |= 2;
 	if (coe_kind == 0) coe_kind = 2;	/* External */
-	
+
 	/* Read the entire data base; note the -I, R and -S options are applied during reading */
-	
-	GMT_Report (API, GMT_MSG_VERBOSE, "Read crossover database %s...\n", Ctrl->In.file);
-	np = x2sys_read_coe_dbase (GMT, s, Ctrl->In.file, Ctrl->I.file, GMT->common.R.wesn, Ctrl->C.col, coe_kind, Ctrl->S.file, &P, &nx, &n_tracks);
-	GMT_Report (API, GMT_MSG_VERBOSE, "Found %" PRIu64 " pairs and a total of %" PRIu64 " crossover records.\n", np, nx);
+
+	GMT_Report (API, GMT_MSG_INFORMATION, "Read crossover database %s...\n", Ctrl->In.file);
+	if ((value = x2sys_read_coe_dbase (GMT, s, Ctrl->In.file, Ctrl->I.file, GMT->common.R.wesn, Ctrl->C.col, coe_kind, Ctrl->S.file, &P, &nx, &n_tracks)) < 0)
+		Return (-value);
+
+	np = (uint64_t)value;
+	GMT_Report (API, GMT_MSG_INFORMATION, "Found %" PRIu64 " pairs and a total of %" PRIu64 " crossover records.\n", np, nx);
 
 	if (np == 0 && nx == 0) {	/* End here since nothing was allocated */
 		x2sys_end (GMT, s);
 		Return (GMT_NOERROR);
 	}
-	
+
 	R = gmt_M_memory (GMT, NULL, n_tracks, struct COE_REPORT);
 	trk_name = gmt_M_memory (GMT, NULL, n_tracks, char *);
-	
+
 	for (p = 0; p < np; p++) {	/* Get track name list */
 		for (k = 0; k < 2; k++) trk_name[P[p].id[k]] = P[p].trk[k];
 	}
-	
+
 	if (Ctrl->L.active) {	/* Load an ephemeral correction table */
-		x2sys_get_corrtable (GMT, s, Ctrl->L.file, n_tracks, trk_name, Ctrl->C.col, NULL, NULL, &CORR);
+		if ((error = x2sys_get_corrtable (GMT, s, Ctrl->L.file, n_tracks, trk_name, Ctrl->C.col, NULL, NULL, &CORR))) {
+			gmt_M_free (GMT, R);	gmt_M_free (GMT, trk_name);
+			Return (error);
+		}
 	}
 
 	Tsum = Tsum2 = 0.0;
@@ -329,7 +341,7 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 			sign = -1.0;
 		}
 	}
-	
+
 	for (k = n_use = 0, sum_w = 0.0; k < n_tracks; k++) {	/* Calculate normalized weights */
 		if (R[k].nx <= Ctrl->N.min) continue;	/* Not enough COEs */
 		if (R[k].nx && R[k].sum2 > 0.0) {
@@ -344,33 +356,42 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 
 	/* Time to issue output */
 
-	if (GMT_Init_IO (API, GMT_IS_TEXTSET, GMT_IS_NONE, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
-		gmt_M_free (GMT, R);
+	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
+		gmt_M_free (GMT, R);	gmt_M_free (GMT, trk_name);
 		Return (API->error);
 	}
-	if (GMT_Begin_IO (API, GMT_IS_TEXTSET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {
-		gmt_M_free (GMT, R);
+	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {
+		gmt_M_free (GMT, R);	gmt_M_free (GMT, trk_name);
 		Return (API->error);	/* Enables data output and sets access mode */
 	}
 	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_NONE) != GMT_NOERROR) {	/* Sets output geometry */
-		gmt_M_free (GMT, R);
+		gmt_M_free (GMT, R);	gmt_M_free (GMT, trk_name);
 		Return (API->error);
 	}
 	gmt_set_tableheader (GMT, GMT_OUT, true);	/* Turn on -ho explicitly */
-	
-	sprintf (record, " Tag: %s %s", Ctrl->T.TAG, Ctrl->C.col);
+
+	sprintf (record, "Tag: %s %s", Ctrl->T.TAG, Ctrl->C.col);
 	GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
-	sprintf (record, " Command: %s", THIS_MODULE_NAME);
-	if (!Ctrl->In.file) strcat (record, " [stdin]");
+	sprintf (record, "Command: %s", THIS_MODULE_CLASSIC_NAME);
+	len = GMT_BUFSIZ - strlen (record) - 1;
+	if (!Ctrl->In.file) {
+		strncat (record, " [stdin]", len);
+		len -= 8;
+	}
 	for (opt = options; opt; opt = opt->next) {
-		strcat (record, " ");
-		if (opt->option == GMT_OPT_INFILE) 
-			strcat (record, opt->arg);
+		strncat (record, " ", len);
+		len--;
+		if (opt->option == GMT_OPT_INFILE) {
+			strncat (record, opt->arg, len);
+			len -= strlen (opt->arg);
+		}
 		else {
 			sprintf (word, "-%c%s", opt->option, opt->arg);
-			strcat (record, word);
+			strncat (record, word, len);
+			len -= strlen (word);
 		}
 	}
+	Out = gmt_new_record (GMT, NULL, record);	/* Since we only need to worry about text in this module */
 	GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
 	sprintf (record, "track%sN%smean%sstdev%srms%sweight[%" PRIu64 "]", c, c, c, c, c, n_use);
 	GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
@@ -381,8 +402,8 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 	         GMT->current.setting.format_float_out,GMT->current.setting.format_float_out,
 	         GMT->current.setting.format_float_out);
 	sprintf (record, fmt, c, Tnx, c, Tmean, c, Tstdev, c, Trms, c);
-	GMT_Put_Record (API, GMT_WRITE_TEXT, record);
-	sprintf (fmt, "%%s%%s%%" PRIu64 "%%s%s%%s%s%%s%s%%s%s\n",
+	GMT_Put_Record (API, GMT_WRITE_DATA, Out);
+	sprintf (fmt, "%%s%%s%%" PRIu64 "%%s%s%%s%s%%s%s%%s%s",
 	         GMT->current.setting.format_float_out,GMT->current.setting.format_float_out,
 	         GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
 	for (k = 0; k < n_tracks; k++) {	/* For each track that generated crossovers */
@@ -392,21 +413,22 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 		R[k].stdev = (R[k].nx > 1) ? sqrt ((R[k].nx * R[k].sum2 - R[k].sum * R[k].sum) / (R[k].nx * (R[k].nx - 1.0))) : GMT->session.d_NaN;
 		R[k].rms = (R[k].nx) ? sqrt (R[k].sum2 / R[k].nx) : GMT->session.d_NaN;
 		sprintf (record, fmt, trk_name[k], c, R[k].nx, c, R[k].mean, c, R[k].stdev, c, R[k].rms, c, R[k].W);
-		GMT_Put_Record (API, GMT_WRITE_TEXT, record);
+		GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 	}
+	gmt_M_free (GMT, Out);
 	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
 		Return (API->error);
 	}
-	
+
 	if (Ctrl->A.active) {	/* Create track adjustment spline files for each track */
 		unsigned int n_out, n1;
-		char file[GMT_BUFSIZ] = {""};
+		char file[PATH_MAX] = {""};
 		double out[2], z[2], z_ij;
 		FILE *fp = NULL;
 		struct COE_ADJLIST *adj = NULL;
 
 		gmt_set_cartesian (GMT, GMT_OUT);	/* Since we will write (dist, COE) pairs */
-		
+
 		adj = gmt_M_memory (GMT, NULL, n_tracks, struct COE_ADJLIST);
 		for (p = 0; p < np; p++) {	/* For each pair of tracks that generated crossovers */
 			for (i = n = 0; i < P[p].nx; i++) {	/* For each COE between this pair */
@@ -436,11 +458,11 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 			adj[k].K[adj[k].n].d = R[k].d_max;			/* Add in anchor point (d_max,0) */
 			adj[k].K[adj[k].n].c = 0.0;
 			adj[k].n++;
-			
-			qsort(adj[k].K, adj[k].n, sizeof(struct COE_ADJUST), comp_structs);
+
+			qsort(adj[k].K, adj[k].n, sizeof(struct COE_ADJUST), x2sysreport_comp_structs);
 			sprintf (file, "%s/%s/%s.%s.adj", X2SYS_HOME, Ctrl->T.TAG, trk_name[k], Ctrl->C.col);
 			if ((fp = gmt_fopen (GMT, file, "w")) == NULL) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Unable to create file %s!\n", file);
+				GMT_Report (API, GMT_MSG_ERROR, "Unable to create file %s!\n", file);
 				/* Free memory before exiting */
 				for (p = k; p < n_tracks; p++)	/* Free everything then bail */
 					gmt_M_free (GMT, adj[k].K);
@@ -460,7 +482,7 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 				n_out++;
 				if (i == n1 || out[0] < adj[k].K[i+1].d) {	/* Time to output */
 					out[1] /= n_out;
-					GMT->current.io.output (GMT, fp, 2, out);
+					gmt_ascii_output_no_text (GMT, fp, 2, out, NULL);
 					out[1] = 0.0;
 					n_out = 0;
 				}
@@ -470,9 +492,9 @@ int GMT_x2sys_report (void *V_API, int mode, void *args) {
 		}
 		gmt_M_free (GMT, adj);
 	}
-	
+
 	/* Done, free up data base array, etc */
-	
+
 	x2sys_free_coe_dbase (GMT, P, np);
 	gmt_M_free (GMT, trk_name);
 	gmt_M_free (GMT, R);
